/* 
 bicycle rear light
 */

#include "p18lf14k50.h"
#include <delays.h>
#include <stdlib.h>
#include <usart.h>
#include ".\sw_spi2.h"
#pragma config FOSC = IRC, PCLKEN = ON, LVP=OFF, BOREN=NOSLP, BORV=19, WDTEN = OFF, HFOFST = OFF, MCLRE=ON, STVREN=ON, DEBUG=ON

/* SPI port */
#define SW_CS_PIN         	PORTCbits.RC5    	// Chip Select ADXL-hez
#define TRIS_SW_CS_PIN    	TRISCbits.TRISC5
#define SW_DIN_PIN        	PORTBbits.RB4  	   	// SPI Data in, az sw_spi.h-ban is definilva!
#define TRIS_SW_DIN_PIN   	TRISBbits.TRISB4
#define SW_DOUT_PIN       	PORTCbits.RC7    	// SPI Data out, az sw_spi.h-ban is definilva!
#define TRIS_SW_DOUT_PIN  	TRISCbits.TRISC7
#define SW_SCK_PIN        	PORTBbits.RB6     	// SPI Clock, az sw_spi.h-ban is definilva!
#define TRIS_SW_SCK_PIN   	TRISBbits.TRISB6

#define ADXL_INT1        	PORTCbits.RC0     	// INT1 a ADXL-tl
#define TRIS_ADXL_INT1   	TRISCbits.TRISC0
#define AWAKE           	PORTCbits.RC2     	// INT2 a ADXL-tl
#define TRIS_AWAKE         	TRISCbits.TRISC2
#define LIGHT_MEASURE      	PORTAbits.RA4     	
#define TRIS_LIGHT_MEASURE 	TRISAbits.TRISA4
#define LIGHT_POWER      	PORTAbits.RA5     	
#define TRIS_LIGHT_POWER 	TRISAbits.TRISA5
#define ADXL_POWER         	PORTBbits.RB7
#define TRIS_ADXL_POWER    	TRISBbits.TRISB7
#define S1               	PORTCbits.RC1
#define TRIS_S1         	TRISCbits.TRISC1
#define LED                 PORTBbits.RB5    
#define TRIS_LED            TRISBbits.TRISB5

void main(void);
void InterruptHandlerHigh(void);
void INT_enable(void);
void ADXLzchar(void);
void portinit(void);
void ADXLinit(void);
void ADXLstatusread(void);
void ADXLreadZ(void);
void ADXLreadZchar(void);
void LIGHT_measure(void);

char Z, break_threshold = 0x28;
unsigned char bcounter = 0x00, lcounter = 0x00, offcounter = 0x00, light_threshold = 0x02; //lth 04
unsigned char gosleep = 0x00;

//	uX.cX.b0=low significant byte;
//	full int=uX.iX;

union 
{

    struct 
    {
        unsigned char b0; // LSB
        unsigned char b1; // MSB
    } cZ;
    int iZ;
} uZ;

void main(void) 
{
    portinit();
    LED=1; 
    Delay1KTCYx(100);
    Delay1KTCYx(100);
    Delay1KTCYx(100);
    LED=0;
    
            
    ADXLinit();
    INT_enable();
    ADXLstatusread();
    Delay1KTCYx(100);
    while (1) 
    {
        /*    
            OpenSWSPI();    
            while(1)
                {
                LIGHT_measure ();
    
                WriteSWSPI 	(ADRESH);
                if(ADRESH>light_threshold)
                {	    WriteSWSPI 	(0xff);}
                Delay1KTCYx(10);      
                }
         */

        LIGHT_measure();

        if (ADRESH < light_threshold) 
        {
            lcounter = 0x20; // adjust here to increase number of flashes after light increased over threshold
        }

        if (bcounter > 0 || lcounter > 0) 
        {
            LED = 1;
        }

        if (lcounter > 0) 
        {
            lcounter = lcounter - 1;
        }

        if (AWAKE == 0) 
        {
            LED = 0;
            Sleep();
            Nop();
        }
        Delay1KTCYx(10);
        LED = 0;
        offcounter = 50;
        ADXLreadZchar();
        if (Z > break_threshold) 
        {
            bcounter = 10;
        }
        while (Z < break_threshold && bcounter < 1 && offcounter > 0) 
        {
            offcounter = offcounter - 1;
            Delay1KTCYx(1);
            ADXLreadZchar();
            if (Z > break_threshold) 
            {
                offcounter = 0;
                bcounter = 10;
            }
        }

        if (bcounter > 0) 
        {
            bcounter = bcounter - 1;
            Delay1KTCYx(10);
        }

    }

    Nop();
}

//----------------------------------------------------------------------------
// High priority interrupt vector

#pragma code InterruptVectorHigh = 0x08

void InterruptVectorHigh(void) 
{
    _asm
            goto InterruptHandlerHigh //jump to interrupt routine
    _endasm
}

//----------------------------------------------------------------------------
// High priority interrupt routine

#pragma code
#pragma interrupt InterruptHandlerHigh

void InterruptHandlerHigh() 
{
    INTCONbits.GIEH = 0;
    if (INTCON3bits.INT2IF) 
    {
        INTCON3bits.INT2IF = 0;     // ADXL
        INTCON3bits.INT1IF = 0;     // button
        Nop();
    }

    if (INTCON3bits.INT1IF) 
    {
        LED = 1;
        INTCON3bits.INT1IF = 0;
        INTCON3bits.INT2IF = 0;
        Delay1KTCYx(200);
        LED = 0;
        if (S1 == 0) 
        {
            if (INTCON3bits.INT2IE == 1) 
            {
                INTCON3bits.INT2IE = 0;
                LED = 0;
                ADXL_POWER = 0;
                LIGHT_MEASURE = 0;
                offcounter = 0;
                bcounter = 0;
//                Delay1KTCYx(10);
                LED=1;              // switch off signal
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(5);
                
                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(5);

                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(6);

                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(7);
                
                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(8);

                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(9);

                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(10);
                
                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(15);
                
                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(20);
                
                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(30);
                
                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(40);
                
                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(50);
                
                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(60);

                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(90);

                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(120);

                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(150);
                
                LED=1;
                Delay1KTCYx(2);
                LED=0;
                Delay1KTCYx(200);
                

                                
                
                LED=1;
                Delay1KTCYx(2);
                LED=0;
                
                
                
                
                
                ANSEL = 0b00000000;
                TRIS_SW_CS_PIN = 1;
                TRIS_SW_DOUT_PIN = 1;
                TRIS_SW_SCK_PIN = 1;
                TRIS_LIGHT_POWER = 1;
                TRIS_ADXL_POWER = 1;
                INTCON3bits.INT1IF = 0;
                INTCON3bits.INT2IF = 0;                
                INTCONbits.GIEH = 1;
                Sleep();
                Nop();
            }

            else
            {
                Delay1KTCYx(100);
                LED=1;
                Delay1KTCYx(50);
                LED=0;
                Reset();
            }
            
   
    
        }

        
        if (INTCON3bits.INT2IE==0)
        {
            LED = 0; 
            INTCON3bits.INT1IF = 0;
            INTCON3bits.INT2IF = 0;
            INTCONbits.GIEH = 1;
            Sleep();
        }
        
        Delay1KTCYx(255);
    }


    INTCON3bits.INT1IF = 0;
    INTCON3bits.INT2IF = 0;
//    INTCONbits.GIEH = 1;
    ADXLinit();
    ADXLstatusread();
    INT_enable();
    Nop();
}

void portinit(void) 
{
    // start 1MHz-en
    OSCCONbits.IRCF2 = 0;
    OSCCONbits.IRCF1 = 1;
    OSCCONbits.IRCF0 = 1;
    OSCCONbits.IDLEN = 0;
    UCONbits.USBEN = 0;
    UCFGbits.UPUEN = 0;
    REFCON0bits.FVR1EN = 0;
    REFCON1bits.D1EN = 0;
    ADCON0 = 0b00000000;
    ADCON1 = 0b00000000;
    WPUB = 0b00000000;
    ANSEL = 0b00000000;
    ANSELH = 0b00000000;
    CM2CON0 = 0b00000000;
    INTCON2bits.RABPU = 1;
    TRIS_SW_CS_PIN = 0;
    TRIS_SW_DIN_PIN = 1;
    TRIS_SW_DOUT_PIN = 0;
    TRIS_SW_SCK_PIN = 0;
    SW_CS_PIN = 1;
    SW_DOUT_PIN = 0;
    SW_SCK_PIN = 0;
    TRIS_ADXL_INT1 = 1;
    TRIS_AWAKE = 1;
    TRIS_LIGHT_MEASURE = 1;
    TRIS_LIGHT_POWER = 0;
    LIGHT_POWER = 0;
    TRIS_ADXL_POWER = 0;
    ADXL_POWER = 1;
    TRIS_S1 = 1;
    TRIS_LED = 0;
    LED = 0;
    INTCONbits.GIE = 0;
    INTCONbits.PEIE = 0;
}

void DelayFor18TCY(void) 
{
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
}

void INT_enable(void) 
{
    INTCONbits.GIE = 0;
    RCONbits.IPEN = 0; // disable priority levels on interrupts
    INTCONbits.PEIE = 0; // disable peripherial interrupts
    INTCONbits.TMR0IE = 0;
    INTCONbits.INT0IE = 0;
    INTCONbits.RABIE = 0;
    INTCON2bits.INTEDG2 = 1; // interrupt on rising edge of AWAKE signal
    INTCON3bits.INT2IP = 1;
    INTCON3bits.INT2IF = 0;
    INTCON3bits.INT2IE = 1; // enable INT2
    INTCON2bits.INTEDG1 = 0; // interrupt on buttonpress
    INTCON3bits.INT1IP = 1;
    INTCON3bits.INT1IF = 0;
    INTCON3bits.INT1IE = 1;



    INTCONbits.GIE = 1;
}

void ADXLinit(void) 
{
    INTCONbits.GIEH = 0;
    LED = 0;
    ADXL_POWER = 0;
    Delay1KTCYx(20);
    TRIS_SW_CS_PIN = 0;
    TRIS_SW_DOUT_PIN = 0;
    TRIS_SW_SCK_PIN = 0;
    ADXL_POWER = 1;
    Delay1KTCYx(20);
    OpenSWSPI();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); // 
    WriteSWSPI(0x1F); // 
    WriteSWSPI(0x52); // 
    SW_CS_PIN = 1;
    Delay1KTCYx(10);
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); // 
    WriteSWSPI(0x20); // 
    WriteSWSPI(0xFA); // 
    SW_CS_PIN = 1;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); //
    WriteSWSPI(0x21); // 
    WriteSWSPI(0x00); // 
    SW_CS_PIN = 1;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); //
    WriteSWSPI(0x22); // activity time register
    WriteSWSPI(0x04); // 
    SW_CS_PIN = 1;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); //
    WriteSWSPI(0x23); // 
    WriteSWSPI(0x96); // 
    SW_CS_PIN = 1;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); //
    WriteSWSPI(0x24); // 
    WriteSWSPI(0x00); // 
    SW_CS_PIN = 1;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); //
    WriteSWSPI(0x25); // inactivity time register, 1/100s
    WriteSWSPI(0x28); // low byte 28
    WriteSWSPI(0x23); // high bite 23
    SW_CS_PIN = 1;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); //	
    WriteSWSPI(0x27); // 
    WriteSWSPI(0x3F); // 
    SW_CS_PIN = 1;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); //
    WriteSWSPI(0x2A); // 
    WriteSWSPI(0x00); // 
    SW_CS_PIN = 1;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); //
    WriteSWSPI(0x2B); // 
    WriteSWSPI(0x40); // 
    SW_CS_PIN = 1;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); //
    WriteSWSPI(0x2C); // 
    WriteSWSPI(0b00010011); // 
    SW_CS_PIN = 1;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0A); //
    WriteSWSPI(0x2D); // 
    WriteSWSPI(0b00010010); // measurement mode, low noise
    SW_CS_PIN = 1;
    INTCONbits.GIEH = 1;
}

void ADXLreadZ(void) 
{
    INTCONbits.GIEH = 0;
    OpenSWSPI();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0B);
    WriteSWSPI(0x12);
    uZ.cZ.b0 = WriteSWSPI(0x00);
    uZ.cZ.b1 = WriteSWSPI(0x00);
    SW_CS_PIN = 1;
    INTCONbits.GIEH = 1;
}

void ADXLstatusread(void) 
{
    INTCONbits.GIEH = 0;
    OpenSWSPI();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0B); // 
    WriteSWSPI(0x0b); // 
    WriteSWSPI(0x00); // 
    SW_CS_PIN = 1;
    INTCONbits.GIEH = 1;
}

void ADXLreadZchar(void) 
{
    INTCONbits.GIEH = 0;
    OpenSWSPI();
    SW_CS_PIN = 0;
    WriteSWSPI(0x0B);
    WriteSWSPI(0x0A);
    Z = WriteSWSPI(0x00);
    SW_CS_PIN = 1;
    INTCONbits.GIEH = 1;
}

void LIGHT_measure(void) 
{
    INTCONbits.GIEH = 0;
    ANSEL = 0b00001000;
    LIGHT_POWER = 1;
    ADCON0 = 0b00001101; // AN3 is the input, switch AD on
    ADCON1 = 0b00000000; // positive reference is Vdd, negative reference is Vss
    ADCON2 = 0b00101111; // left justified, ACQT = 12 Tad, ADSC = Frc
    Delay10TCYx(25); // delay needed to settle voltage level
    ADCON0bits.GO_DONE = 1;
    while (ADCON0bits.GO_DONE) 
    {
        Nop();
    }
    LIGHT_POWER = 0;
    ADCON0 = 0b00000000;
    ADCON1 = 0b00000000;
    ADCON2 = 0b00000000;
    ANSEL = 0b00000000;
    INTCONbits.GIEH = 1;
}